@@ -1,21 +1,22 @@
 # __init__.py -- The tests for dulwich
 # Copyright (C) 2007 James Westby <jw+debian@jameswestby.net>
 #
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License
-# as published by the Free Software Foundation; version 2
-# of the License or (at your option) any later version of
-# the License.
+# Dulwich is dual-licensed under the Apache License, Version 2.0 and the GNU
+# General Public License as public by the Free Software Foundation; version 2.0
+# or (at your option) any later version. You can redistribute it and/or
+# modify it under the terms of either of these two licenses.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+# You should have received a copy of the licenses; if not, see
+# <http://www.gnu.org/licenses/> for a copy of the GNU General Public License
+# and <http://www.apache.org/licenses/LICENSE-2.0> for a copy of the Apache
+# License, Version 2.0.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-# MA  02110-1301, USA.
 
 """Tests for Dulwich."""
 
@@ -29,55 +30,36 @@ import tempfile
 
 # If Python itself provides an exception, use that
 import unittest
-if sys.version_info < (2, 7):
-    from unittest2 import SkipTest, TestCase as _TestCase, skipIf, expectedFailure
-else:
-    from unittest import SkipTest, TestCase as _TestCase, skipIf, expectedFailure
-
-
-def get_safe_env(env=None):
-    """Returns the environment "env" (or a copy of "os.environ" by default)
-    modified to avoid side-effects caused by user's ~/.gitconfig"""
-
-    if env is None:
-        env = os.environ.copy()
-    # On Windows it's not enough to set "HOME" to a non-existing
-    # directory. Git.cmd takes the first existing directory out of
-    # "%HOME%", "%HOMEDRIVE%%HOMEPATH%" and "%USERPROFILE%".
-    for e in 'HOME', 'HOMEPATH', 'USERPROFILE':
-        env[e] = '/nosuchdir'
-    return env
+from unittest import (  # noqa: F401
+    SkipTest,
+    TestCase as _TestCase,
+    skipIf,
+    expectedFailure,
+    )
 
 
 class TestCase(_TestCase):
 
-    def makeSafeEnv(self):
-        """Create environment with homedirectory-related variables stripped.
-
-        Modifies os.environ for the duration of a test case to avoid
-        side-effects caused by the user's ~/.gitconfig and other
-        files in their home directory.
-        """
-        old_env = os.environ
-        def restore():
-            os.environ = old_env
-        self.addCleanup(restore)
-        new_env = dict(os.environ)
-        for e in ['HOME', 'HOMEPATH', 'USERPROFILE']:
-            new_env[e] = '/nosuchdir'
-        os.environ = new_env
-
     def setUp(self):
         super(TestCase, self).setUp()
-        self.makeSafeEnv()
+        self._old_home = os.environ.get("HOME")
+        os.environ["HOME"] = "/nonexistant"
+
+    def tearDown(self):
+        super(TestCase, self).tearDown()
+        if self._old_home:
+            os.environ["HOME"] = self._old_home
+        else:
+            del os.environ["HOME"]
 
 
 class BlackboxTestCase(TestCase):
     """Blackbox testing."""
 
     # TODO(jelmer): Include more possible binary paths.
-    bin_directories = [os.path.abspath(os.path.join(os.path.dirname(__file__),
-        "..", "..", "bin")), '/usr/bin', '/usr/local/bin']
+    bin_directories = [os.path.abspath(os.path.join(
+            os.path.dirname(__file__), "..", "..", "bin")), '/usr/bin',
+            '/usr/local/bin']
 
     def bin_path(self, name):
         """Determine the full path of a binary.
@@ -107,10 +89,11 @@ class BlackboxTestCase(TestCase):
         #
         # Save us from all that headache and call python with the bin script.
         argv = [sys.executable, self.bin_path(name)] + args
-        return subprocess.Popen(argv,
-            stdout=subprocess.PIPE,
-            stdin=subprocess.PIPE, stderr=subprocess.PIPE,
-            env=env)
+        return subprocess.Popen(
+                argv,
+                stdout=subprocess.PIPE,
+                stdin=subprocess.PIPE, stderr=subprocess.PIPE,
+                env=env)
 
 
 def self_test_suite():
@@ -125,6 +108,7 @@ def self_test_suite():
         'grafts',
         'greenthreads',
         'hooks',
+        'ignore',
         'index',
         'lru_cache',
         'objects',
@@ -139,6 +123,7 @@ def self_test_suite():
         'refs',
         'repository',
         'server',
+        'utils',
         'walk',
         'web',
         ]
@@ -157,15 +142,17 @@ def tutorial_test_suite():
         'conclusion',
         ]
     tutorial_files = ["../../docs/tutorial/%s.txt" % name for name in tutorial]
+
     def setup(test):
         test.__old_cwd = os.getcwd()
         test.__dulwich_tempdir = tempfile.mkdtemp()
         os.chdir(test.__dulwich_tempdir)
+
     def teardown(test):
         os.chdir(test.__old_cwd)
         shutil.rmtree(test.__dulwich_tempdir)
-    return doctest.DocFileSuite(setUp=setup, tearDown=teardown,
-        *tutorial_files)
+    return doctest.DocFileSuite(
+            setUp=setup, tearDown=teardown, *tutorial_files)
 
 
 def nocompat_test_suite():
