@@ -1,20 +1,22 @@
 # test_diff_tree.py -- Tests for file and tree diff utilities.
 # Copyright (C) 2010 Google, Inc.
 #
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License
-# as published by the Free Software Foundation; either version 2
-# or (at your option) a later version of the License.
+# Dulwich is dual-licensed under the Apache License, Version 2.0 and the GNU
+# General Public License as public by the Free Software Foundation; version 2.0
+# or (at your option) any later version. You can redistribute it and/or
+# modify it under the terms of either of these two licenses.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+# You should have received a copy of the licenses; if not, see
+# <http://www.gnu.org/licenses/> for a copy of the GNU General Public License
+# and <http://www.apache.org/licenses/LICENSE-2.0> for a copy of the Apache
+# License, Version 2.0.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-# MA  02110-1301, USA.
 
 """Tests for file and tree diff utilities."""
 
@@ -169,7 +171,8 @@ class TreeChangesTest(DiffTestCase):
         self.assertChangesEqual([], self.empty_tree, self.empty_tree)
         self.assertChangesEqual([], tree, tree)
         self.assertChangesEqual(
-            [TreeChange(CHANGE_UNCHANGED, (b'a', F, blob.id), (b'a', F, blob.id)),
+            [TreeChange(CHANGE_UNCHANGED, (b'a', F, blob.id),
+                        (b'a', F, blob.id)),
              TreeChange(CHANGE_UNCHANGED, (b'b/c', F, blob.id),
                         (b'b/c', F, blob.id))],
             tree, tree, want_unchanged=True)
@@ -267,7 +270,8 @@ class TreeChangesTest(DiffTestCase):
         tree1 = self.commit_tree([(b'a', blob), (b'a.', blob), (b'a..', blob)])
         # Tree order is the reverse of this, so if we used tree order, 'a..'
         # would not be merged.
-        tree2 = self.commit_tree([(b'a/x', blob), (b'a./x', blob), (b'a..', blob)])
+        tree2 = self.commit_tree(
+                [(b'a/x', blob), (b'a./x', blob), (b'a..', blob)])
 
         self.assertChangesEqual(
             [TreeChange.delete((b'a', F, blob.id)),
@@ -350,7 +354,8 @@ class TreeChangesTest(DiffTestCase):
         merge = self.commit_tree([(b'a', blob2)])
         self.assertChangesForMergeEqual(
             [[TreeChange.add((b'a', F, blob2.id)),
-              TreeChange(CHANGE_MODIFY, (b'a', F, blob1.id), (b'a', F, blob2.id))]],
+              TreeChange(CHANGE_MODIFY, (b'a', F, blob1.id),
+                         (b'a', F, blob2.id))]],
             [parent1, parent2], merge)
 
     def test_tree_changes_for_merge_modify_modify_conflict(self):
@@ -361,8 +366,10 @@ class TreeChangesTest(DiffTestCase):
         parent2 = self.commit_tree([(b'a', blob2)])
         merge = self.commit_tree([(b'a', blob3)])
         self.assertChangesForMergeEqual(
-            [[TreeChange(CHANGE_MODIFY, (b'a', F, blob1.id), (b'a', F, blob3.id)),
-              TreeChange(CHANGE_MODIFY, (b'a', F, blob2.id), (b'a', F, blob3.id))]],
+            [[TreeChange(CHANGE_MODIFY, (b'a', F, blob1.id),
+                         (b'a', F, blob3.id)),
+              TreeChange(CHANGE_MODIFY, (b'a', F, blob2.id),
+                         (b'a', F, blob3.id))]],
             [parent1, parent2], merge)
 
     def test_tree_changes_for_merge_modify_no_conflict(self):
@@ -432,7 +439,8 @@ class TreeChangesTest(DiffTestCase):
         parent2 = self.commit_tree([])
         merge = self.commit_tree([(b'b', blob)])
         add = TreeChange.add((b'b', F, blob.id))
-        self.assertChangesForMergeEqual([[add, add]], [parent1, parent2], merge)
+        self.assertChangesForMergeEqual(
+                [[add, add]], [parent1, parent2], merge)
 
     def test_tree_changes_for_merge_add_exact_rename_conflict(self):
         blob = make_object(Blob, data=b'a\nb\nc\nd\n')
@@ -440,7 +448,8 @@ class TreeChangesTest(DiffTestCase):
         parent2 = self.commit_tree([])
         merge = self.commit_tree([(b'b', blob)])
         self.assertChangesForMergeEqual(
-            [[TreeChange(CHANGE_RENAME, (b'a', F, blob.id), (b'b', F, blob.id)),
+            [[TreeChange(CHANGE_RENAME, (b'a', F, blob.id),
+                         (b'b', F, blob.id)),
               TreeChange.add((b'b', F, blob.id))]],
             [parent1, parent2], merge, rename_detector=self.detector)
 
@@ -451,7 +460,8 @@ class TreeChangesTest(DiffTestCase):
         parent2 = self.commit_tree([])
         merge = self.commit_tree([(b'b', blob2)])
         self.assertChangesForMergeEqual(
-            [[TreeChange(CHANGE_RENAME, (b'a', F, blob1.id), (b'b', F, blob2.id)),
+            [[TreeChange(CHANGE_RENAME, (b'a', F, blob1.id),
+                         (b'b', F, blob2.id)),
               TreeChange.add((b'b', F, blob2.id))]],
             [parent1, parent2], merge, rename_detector=self.detector)
 
@@ -462,8 +472,10 @@ class TreeChangesTest(DiffTestCase):
         parent2 = self.commit_tree([(b'b', blob1)])
         merge = self.commit_tree([(b'b', blob2)])
         self.assertChangesForMergeEqual(
-            [[TreeChange(CHANGE_RENAME, (b'a', F, blob1.id), (b'b', F, blob2.id)),
-              TreeChange(CHANGE_MODIFY, (b'b', F, blob1.id), (b'b', F, blob2.id))]],
+            [[TreeChange(CHANGE_RENAME, (b'a', F, blob1.id),
+                         (b'b', F, blob2.id)),
+              TreeChange(CHANGE_MODIFY, (b'b', F, blob1.id),
+                         (b'b', F, blob2.id))]],
             [parent1, parent2], merge, rename_detector=self.detector)
 
 
@@ -471,7 +483,8 @@ class RenameDetectionTest(DiffTestCase):
 
     def _do_test_count_blocks(self, count_blocks):
         blob = make_object(Blob, data=b'a\nb\na\n')
-        self.assertEqual({hash(b'a\n'): 4, hash(b'b\n'): 2}, count_blocks(blob))
+        self.assertEqual({hash(b'a\n'): 4, hash(b'b\n'): 2},
+                         count_blocks(blob))
 
     test_count_blocks = functest_builder(_do_test_count_blocks,
                                          _count_blocks_py)
@@ -489,7 +502,8 @@ class RenameDetectionTest(DiffTestCase):
 
     def _do_test_count_blocks_chunks(self, count_blocks):
         blob = ShaFile.from_raw_chunks(Blob.type_num, [b'a\nb', b'\na\n'])
-        self.assertEqual({hash(b'a\n'): 4, hash(b'b\n'): 2}, _count_blocks(blob))
+        self.assertEqual({hash(b'a\n'): 4, hash(b'b\n'): 2},
+                         _count_blocks(blob))
 
     test_count_blocks_chunks = functest_builder(_do_test_count_blocks_chunks,
                                                 _count_blocks_py)
@@ -500,8 +514,8 @@ class RenameDetectionTest(DiffTestCase):
         a = b'a' * 64
         data = a + b'xxx\ny\n' + a + b'zzz\n'
         blob = make_object(Blob, data=data)
-        self.assertEqual({hash(b'a' * 64): 128, hash(b'xxx\n'): 4, hash(b'y\n'): 2,
-                          hash(b'zzz\n'): 4},
+        self.assertEqual({hash(b'a' * 64): 128, hash(b'xxx\n'): 4,
+                          hash(b'y\n'): 2, hash(b'zzz\n'): 4},
                          _count_blocks(blob))
 
     test_count_blocks_long_lines = functest_builder(
@@ -575,7 +589,8 @@ class RenameDetectionTest(DiffTestCase):
         tree1 = self.commit_tree([(b'a', blob1), (b'b', blob2)])
         tree2 = self.commit_tree([(b'a', blob1), (b'b', blob3)])
         self.assertEqual(
-            [TreeChange(CHANGE_MODIFY, (b'b', F, blob2.id), (b'b', F, blob3.id))],
+            [TreeChange(CHANGE_MODIFY, (b'b', F, blob2.id),
+                        (b'b', F, blob3.id))],
             self.detect_renames(tree1, tree2))
 
     def test_exact_rename_one_to_one(self):
@@ -584,9 +599,11 @@ class RenameDetectionTest(DiffTestCase):
         tree1 = self.commit_tree([(b'a', blob1), (b'b', blob2)])
         tree2 = self.commit_tree([(b'c', blob1), (b'd', blob2)])
         self.assertEqual(
-            [TreeChange(CHANGE_RENAME, (b'a', F, blob1.id), (b'c', F, blob1.id)),
-             TreeChange(CHANGE_RENAME, (b'b', F, blob2.id), (b'd', F, blob2.id))],
-            self.detect_renames(tree1, tree2))
+                [TreeChange(CHANGE_RENAME, (b'a', F, blob1.id),
+                            (b'c', F, blob1.id)),
+                 TreeChange(CHANGE_RENAME, (b'b', F, blob2.id),
+                            (b'd', F, blob2.id))],
+                self.detect_renames(tree1, tree2))
 
     def test_exact_rename_split_different_type(self):
         blob = make_object(Blob, data=b'/foo')
@@ -603,9 +620,10 @@ class RenameDetectionTest(DiffTestCase):
         tree1 = self.commit_tree([(b'a', blob1)])
         tree2 = self.commit_tree([(b'a', blob2, 0o120000), (b'b', blob1)])
         self.assertEqual(
-            [TreeChange.add((b'a', 0o120000, blob2.id)),
-             TreeChange(CHANGE_RENAME, (b'a', F, blob1.id), (b'b', F, blob1.id))],
-            self.detect_renames(tree1, tree2))
+                [TreeChange.add((b'a', 0o120000, blob2.id)),
+                 TreeChange(CHANGE_RENAME, (b'a', F, blob1.id),
+                            (b'b', F, blob1.id))],
+                self.detect_renames(tree1, tree2))
 
     def test_exact_rename_one_to_many(self):
         blob = make_object(Blob, data=b'1')
@@ -630,10 +648,13 @@ class RenameDetectionTest(DiffTestCase):
         tree1 = self.commit_tree([(b'a', blob), (b'b', blob)])
         tree2 = self.commit_tree([(b'c', blob), (b'd', blob), (b'e', blob)])
         self.assertEqual(
-            [TreeChange(CHANGE_RENAME, (b'a', F, blob.id), (b'c', F, blob.id)),
-             TreeChange(CHANGE_COPY, (b'a', F, blob.id), (b'e', F, blob.id)),
-             TreeChange(CHANGE_RENAME, (b'b', F, blob.id), (b'd', F, blob.id))],
-            self.detect_renames(tree1, tree2))
+                [TreeChange(CHANGE_RENAME, (b'a', F, blob.id),
+                            (b'c', F, blob.id)),
+                 TreeChange(CHANGE_COPY, (b'a', F, blob.id),
+                            (b'e', F, blob.id)),
+                 TreeChange(CHANGE_RENAME, (b'b', F, blob.id),
+                            (b'd', F, blob.id))],
+                self.detect_renames(tree1, tree2))
 
     def test_exact_copy_modify(self):
         blob1 = make_object(Blob, data=b'a\nb\nc\nd\n')
@@ -641,8 +662,10 @@ class RenameDetectionTest(DiffTestCase):
         tree1 = self.commit_tree([(b'a', blob1)])
         tree2 = self.commit_tree([(b'a', blob2), (b'b', blob1)])
         self.assertEqual(
-            [TreeChange(CHANGE_MODIFY, (b'a', F, blob1.id), (b'a', F, blob2.id)),
-             TreeChange(CHANGE_COPY, (b'a', F, blob1.id), (b'b', F, blob1.id))],
+            [TreeChange(CHANGE_MODIFY, (b'a', F, blob1.id),
+                        (b'a', F, blob2.id)),
+             TreeChange(CHANGE_COPY, (b'a', F, blob1.id),
+                        (b'b', F, blob1.id))],
             self.detect_renames(tree1, tree2))
 
     def test_exact_copy_change_mode(self):
@@ -661,7 +684,8 @@ class RenameDetectionTest(DiffTestCase):
         tree1 = self.commit_tree([(b'a', blob1)])
         tree2 = self.commit_tree([(b'b', blob2)])
         self.assertEqual(
-            [TreeChange(CHANGE_RENAME, (b'a', F, blob1.id), (b'b', F, blob2.id))],
+            [TreeChange(CHANGE_RENAME, (b'a', F, blob1.id),
+                        (b'b', F, blob2.id))],
             self.detect_renames(tree1, tree2, rename_threshold=50))
         self.assertEqual(
             [TreeChange.delete((b'a', F, blob1.id)),
@@ -676,8 +700,10 @@ class RenameDetectionTest(DiffTestCase):
         tree1 = self.commit_tree([(b'a', blob1), (b'b', blob2)])
         tree2 = self.commit_tree([(b'c', blob3), (b'd', blob4)])
         self.assertEqual(
-            [TreeChange(CHANGE_RENAME, (b'a', F, blob1.id), (b'd', F, blob4.id)),
-             TreeChange(CHANGE_RENAME, (b'b', F, blob2.id), (b'c', F, blob3.id))],
+            [TreeChange(CHANGE_RENAME, (b'a', F, blob1.id),
+                        (b'd', F, blob4.id)),
+             TreeChange(CHANGE_RENAME, (b'b', F, blob2.id),
+                        (b'c', F, blob3.id))],
             self.detect_renames(tree1, tree2))
         self.assertEqual(
             [TreeChange.delete((b'a', F, blob1.id)),
@@ -707,13 +733,15 @@ class RenameDetectionTest(DiffTestCase):
         tree2 = self.commit_tree([(b'c', blob3)])
         self.assertEqual(
             [TreeChange.delete((b'a', F, blob1.id)),
-             TreeChange(CHANGE_RENAME, (b'b', F, blob2.id), (b'c', F, blob3.id))],
+             TreeChange(CHANGE_RENAME, (b'b', F, blob2.id),
+                        (b'c', F, blob3.id))],
             self.detect_renames(tree1, tree2))
 
         tree3 = self.commit_tree([(b'a', blob2), (b'b', blob1)])
         tree4 = self.commit_tree([(b'c', blob3)])
         self.assertEqual(
-            [TreeChange(CHANGE_RENAME, (b'a', F, blob2.id), (b'c', F, blob3.id)),
+            [TreeChange(CHANGE_RENAME, (b'a', F, blob2.id),
+                        (b'c', F, blob3.id)),
              TreeChange.delete((b'b', F, blob1.id))],
             self.detect_renames(tree3, tree4))
 
@@ -725,7 +753,8 @@ class RenameDetectionTest(DiffTestCase):
         tree2 = self.commit_tree([(b'b', blob2), (b'c', blob3)])
         self.assertEqual(
             [TreeChange(CHANGE_COPY, (b'a', F, blob1.id), (b'b', F, blob2.id)),
-             TreeChange(CHANGE_RENAME, (b'a', F, blob1.id), (b'c', F, blob3.id))],
+             TreeChange(CHANGE_RENAME, (b'a', F, blob1.id),
+                        (b'c', F, blob3.id))],
             self.detect_renames(tree1, tree2))
 
     def test_content_rename_many_to_one(self):
@@ -735,7 +764,8 @@ class RenameDetectionTest(DiffTestCase):
         tree1 = self.commit_tree([(b'a', blob1), (b'b', blob2)])
         tree2 = self.commit_tree([(b'c', blob3)])
         self.assertEqual(
-            [TreeChange(CHANGE_RENAME, (b'a', F, blob1.id), (b'c', F, blob3.id)),
+            [TreeChange(CHANGE_RENAME, (b'a', F, blob1.id),
+                        (b'c', F, blob3.id)),
              TreeChange.delete((b'b', F, blob2.id))],
             self.detect_renames(tree1, tree2))
 
@@ -749,7 +779,8 @@ class RenameDetectionTest(DiffTestCase):
         # TODO(dborowitz): Distribute renames rather than greedily choosing
         # copies.
         self.assertEqual(
-            [TreeChange(CHANGE_RENAME, (b'a', F, blob1.id), (b'c', F, blob3.id)),
+            [TreeChange(CHANGE_RENAME, (b'a', F, blob1.id),
+                        (b'c', F, blob3.id)),
              TreeChange(CHANGE_COPY, (b'a', F, blob1.id), (b'd', F, blob4.id)),
              TreeChange.delete((b'b', F, blob2.id))],
             self.detect_renames(tree1, tree2))
@@ -787,12 +818,16 @@ class RenameDetectionTest(DiffTestCase):
         tree1 = self.commit_tree([(b'a', blob1), (b'b', blob2)])
         tree2 = self.commit_tree([(b'a', blob2), (b'b', blob1)])
         self.assertEqual(
-            [TreeChange(CHANGE_MODIFY, (b'a', F, blob1.id), (b'a', F, blob2.id)),
-             TreeChange(CHANGE_MODIFY, (b'b', F, blob2.id), (b'b', F, blob1.id))],
+            [TreeChange(CHANGE_MODIFY, (b'a', F, blob1.id),
+                        (b'a', F, blob2.id)),
+             TreeChange(CHANGE_MODIFY, (b'b', F, blob2.id),
+                        (b'b', F, blob1.id))],
             self.detect_renames(tree1, tree2))
         self.assertEqual(
-            [TreeChange(CHANGE_RENAME, (b'a', F, blob1.id), (b'b', F, blob1.id)),
-             TreeChange(CHANGE_RENAME, (b'b', F, blob2.id), (b'a', F, blob2.id))],
+            [TreeChange(CHANGE_RENAME, (b'a', F, blob1.id),
+                        (b'b', F, blob1.id)),
+             TreeChange(CHANGE_RENAME, (b'b', F, blob2.id),
+                        (b'a', F, blob2.id))],
             self.detect_renames(tree1, tree2, rewrite_threshold=50))
 
     def test_content_rename_swap(self):
@@ -803,8 +838,10 @@ class RenameDetectionTest(DiffTestCase):
         tree1 = self.commit_tree([(b'a', blob1), (b'b', blob2)])
         tree2 = self.commit_tree([(b'a', blob4), (b'b', blob3)])
         self.assertEqual(
-            [TreeChange(CHANGE_RENAME, (b'a', F, blob1.id), (b'b', F, blob3.id)),
-             TreeChange(CHANGE_RENAME, (b'b', F, blob2.id), (b'a', F, blob4.id))],
+            [TreeChange(CHANGE_RENAME, (b'a', F, blob1.id),
+                        (b'b', F, blob3.id)),
+             TreeChange(CHANGE_RENAME, (b'b', F, blob2.id),
+                        (b'a', F, blob4.id))],
             self.detect_renames(tree1, tree2, rewrite_threshold=60))
 
     def test_rewrite_threshold(self):
@@ -816,15 +853,18 @@ class RenameDetectionTest(DiffTestCase):
         tree2 = self.commit_tree([(b'a', blob3), (b'b', blob2)])
 
         no_renames = [
-            TreeChange(CHANGE_MODIFY, (b'a', F, blob1.id), (b'a', F, blob3.id)),
+            TreeChange(CHANGE_MODIFY, (b'a', F, blob1.id),
+                       (b'a', F, blob3.id)),
             TreeChange(CHANGE_COPY, (b'a', F, blob1.id), (b'b', F, blob2.id))]
         self.assertEqual(
             no_renames, self.detect_renames(tree1, tree2))
         self.assertEqual(
-            no_renames, self.detect_renames(tree1, tree2, rewrite_threshold=40))
+            no_renames, self.detect_renames(
+                tree1, tree2, rewrite_threshold=40))
         self.assertEqual(
             [TreeChange.add((b'a', F, blob3.id)),
-             TreeChange(CHANGE_RENAME, (b'a', F, blob1.id), (b'b', F, blob2.id))],
+             TreeChange(CHANGE_RENAME, (b'a', F, blob1.id),
+                        (b'b', F, blob2.id))],
             self.detect_renames(tree1, tree2, rewrite_threshold=80))
 
     def test_find_copies_harder_exact(self):
@@ -845,7 +885,8 @@ class RenameDetectionTest(DiffTestCase):
         self.assertEqual([TreeChange.add((b'b', F, blob2.id))],
                          self.detect_renames(tree1, tree2))
         self.assertEqual(
-            [TreeChange(CHANGE_COPY, (b'a', F, blob1.id), (b'b', F, blob2.id))],
+            [TreeChange(CHANGE_COPY, (b'a', F, blob1.id),
+                        (b'b', F, blob2.id))],
             self.detect_renames(tree1, tree2, find_copies_harder=True))
 
     def test_find_copies_harder_with_rewrites(self):
@@ -857,7 +898,8 @@ class RenameDetectionTest(DiffTestCase):
         self.assertEqual(
             [TreeChange(CHANGE_MODIFY, (b'a', F, blob_a1.id),
                         (b'a', F, blob_a2.id)),
-             TreeChange(CHANGE_COPY, (b'a', F, blob_a1.id), (b'b', F, blob_b2.id))],
+             TreeChange(CHANGE_COPY, (b'a', F, blob_a1.id),
+                        (b'b', F, blob_b2.id))],
             self.detect_renames(tree1, tree2, find_copies_harder=True))
         self.assertEqual(
             [TreeChange.add((b'a', F, blob_a2.id)),
