@@ -2,20 +2,22 @@
 # Copyright (C) 2007 James Westby <jw+debian@jameswestby.net>
 # Copyright (C) 2008 Jelmer Vernooij <jelmer@samba.org>
 #
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License
-# as published by the Free Software Foundation; version 2
-# of the License, or (at your option) any later version of the license.
+# Dulwich is dual-licensed under the Apache License, Version 2.0 and the GNU
+# General Public License as public by the Free Software Foundation; version 2.0
+# or (at your option) any later version. You can redistribute it and/or
+# modify it under the terms of either of these two licenses.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+# You should have received a copy of the licenses; if not, see
+# <http://www.gnu.org/licenses/> for a copy of the GNU General Public License
+# and <http://www.apache.org/licenses/LICENSE-2.0> for a copy of the Apache
+# License, Version 2.0.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-# MA  02110-1301, USA.
 
 """Tests for Dulwich packs."""
 
@@ -91,19 +93,24 @@ class PackTests(TestCase):
         self.tempdir = tempfile.mkdtemp()
         self.addCleanup(shutil.rmtree, self.tempdir)
 
-    datadir = os.path.abspath(os.path.join(os.path.dirname(__file__),
-        'data/packs'))
+    datadir = os.path.abspath(
+            os.path.join(os.path.dirname(__file__), 'data/packs'))
 
     def get_pack_index(self, sha):
         """Returns a PackIndex from the datadir with the given sha"""
-        return load_pack_index(os.path.join(self.datadir, 'pack-%s.idx' % sha.decode('ascii')))
+        return load_pack_index(
+                os.path.join(self.datadir,
+                             'pack-%s.idx' % sha.decode('ascii')))
 
     def get_pack_data(self, sha):
         """Returns a PackData object from the datadir with the given sha"""
-        return PackData(os.path.join(self.datadir, 'pack-%s.pack' % sha.decode('ascii')))
+        return PackData(
+                os.path.join(
+                        self.datadir, 'pack-%s.pack' % sha.decode('ascii')))
 
     def get_pack(self, sha):
-        return Pack(os.path.join(self.datadir, 'pack-%s' % sha.decode('ascii')))
+        return Pack(
+                os.path.join(self.datadir, 'pack-%s' % sha.decode('ascii')))
 
     def assertSucceeds(self, func, *args, **kwargs):
         try:
@@ -163,8 +170,9 @@ class TestPackDeltas(TestCase):
     test_string_huge = b'Z' * 100000
 
     def _test_roundtrip(self, base, target):
-        self.assertEqual(target,
-                          b''.join(apply_delta(base, create_delta(base, target))))
+        self.assertEqual(
+                target,
+                b''.join(apply_delta(base, create_delta(base, target))))
 
     def test_nochange(self):
         self._test_roundtrip(self.test_string1, self.test_string1)
@@ -189,13 +197,50 @@ class TestPackDeltas(TestCase):
                              self.test_string_huge + self.test_string2)
 
     def test_dest_overflow(self):
-        self.assertRaises(
-            ApplyDeltaError,
-            apply_delta, b'a'*0x10000, b'\x80\x80\x04\x80\x80\x04\x80' + b'a'*0x10000)
+        self.assertRaises(ApplyDeltaError, apply_delta,
+                          b'a'*0x10000, b'\x80\x80\x04\x80\x80\x04\x80' +
+                          b'a'*0x10000)
         self.assertRaises(
             ApplyDeltaError,
             apply_delta, b'', b'\x00\x80\x02\xb0\x11\x11')
 
+    def test_pypy_issue(self):
+        # Test for https://github.com/jelmer/dulwich/issues/509 /
+        # https://bitbucket.org/pypy/pypy/issues/2499/cpyext-pystring_asstring-doesnt-work
+        chunks = [
+            b'tree 03207ccf58880a748188836155ceed72f03d65d6\n'
+            b'parent 408fbab530fd4abe49249a636a10f10f44d07a21\n'
+            b'author Victor Stinner <victor.stinner@gmail.com> '
+            b'1421355207 +0100\n'
+            b'committer Victor Stinner <victor.stinner@gmail.com> '
+            b'1421355207 +0100\n'
+            b'\n'
+            b'Backout changeset 3a06020af8cf\n'
+            b'\nStreamWriter: close() now clears the reference to the '
+            b'transport\n'
+            b'\nStreamWriter now raises an exception if it is closed: '
+            b'write(), writelines(),\n'
+            b'write_eof(), can_write_eof(), get_extra_info(), drain().\n']
+        delta = [
+            b'\xcd\x03\xad\x03]tree ff3c181a393d5a7270cddc01ea863818a8621ca8\n'
+            b'parent 20a103cc90135494162e819f98d0edfc1f1fba6b\x91]7\x0510738'
+            b'\x91\x99@\x0b10738 +0100\x93\x04\x01\xc9']
+        res = apply_delta(chunks, delta)
+        expected = [
+            b'tree ff3c181a393d5a7270cddc01ea863818a8621ca8\n'
+            b'parent 20a103cc90135494162e819f98d0edfc1f1fba6b',
+            b'\nauthor Victor Stinner <victor.stinner@gmail.com> 14213',
+            b'10738',
+            b' +0100\ncommitter Victor Stinner <victor.stinner@gmail.com> '
+            b'14213',
+            b'10738 +0100',
+            b'\n\nStreamWriter: close() now clears the reference to the '
+            b'transport\n\n'
+            b'StreamWriter now raises an exception if it is closed: '
+            b'write(), writelines(),\n'
+            b'write_eof(), can_write_eof(), get_extra_info(), drain().\n']
+        self.assertEqual(b''.join(expected), b''.join(res))
+
 
 class TestPackData(PackTests):
     """Tests getting the data from the packfile."""
@@ -204,7 +249,8 @@ class TestPackData(PackTests):
         self.get_pack_data(pack1_sha).close()
 
     def test_from_file(self):
-        path = os.path.join(self.datadir, 'pack-%s.pack' % pack1_sha.decode('ascii'))
+        path = os.path.join(self.datadir,
+                            'pack-%s.pack' % pack1_sha.decode('ascii'))
         with open(path, 'rb') as f:
             PackData.from_file(f, os.path.getsize(path))
 
@@ -218,13 +264,14 @@ class TestPackData(PackTests):
 
     def test_iterobjects(self):
         with self.get_pack_data(pack1_sha) as p:
-            commit_data = (b'tree b2a2766a2879c209ab1176e7e778b81ae422eeaa\n'
-                           b'author James Westby <jw+debian@jameswestby.net> '
-                           b'1174945067 +0100\n'
-                           b'committer James Westby <jw+debian@jameswestby.net> '
-                           b'1174945067 +0100\n'
-                           b'\n'
-                           b'Test commit\n')
+            commit_data = (
+                    b'tree b2a2766a2879c209ab1176e7e778b81ae422eeaa\n'
+                    b'author James Westby <jw+debian@jameswestby.net> '
+                    b'1174945067 +0100\n'
+                    b'committer James Westby <jw+debian@jameswestby.net> '
+                    b'1174945067 +0100\n'
+                    b'\n'
+                    b'Test commit\n')
             blob_sha = b'6f670c0fb53f9463760b7295fbb814e965fb20c8'
             tree_data = b'100644 a\0' + hex_to_sha(blob_sha)
             actual = []
@@ -280,7 +327,7 @@ class TestPackData(PackTests):
         self.assertRaises(AssertionError, compute_file_sha, f, end_ofs=-20)
         self.assertRaises(AssertionError, compute_file_sha, f, end_ofs=20)
         self.assertRaises(AssertionError, compute_file_sha, f, start_ofs=10,
-            end_ofs=-12)
+                          end_ofs=-12)
 
 
 class TestPack(PackTests):
@@ -309,7 +356,8 @@ class TestPack(PackTests):
     def test_pack_tuples(self):
         with self.get_pack(pack1_sha) as p:
             tuples = p.pack_tuples()
-            expected = set([(p[s], None) for s in [commit_sha, tree_sha, a_sha]])
+            expected = set(
+                    [(p[s], None) for s in [commit_sha, tree_sha, a_sha]])
             self.assertEqual(expected, set(list(tuples)))
             self.assertEqual(expected, set(list(tuples)))
             self.assertEqual(3, len(tuples))
@@ -338,7 +386,7 @@ class TestPack(PackTests):
                 self.assertSucceeds(newpack.index.check)
                 self.assertEqual(origpack.name(), newpack.name())
                 self.assertEqual(origpack.index.get_pack_checksum(),
-                                  newpack.index.get_pack_checksum())
+                                 newpack.index.get_pack_checksum())
 
                 wrong_version = origpack.index.version != newpack.index.version
                 orig_checksum = origpack.index.get_stored_checksum()
@@ -545,10 +593,10 @@ class BaseTestPackIndexWriting(object):
         entry1_sha = hex_to_sha('4e6388232ec39792661e2e75db8fb117fc869ce6')
         entry2_sha = hex_to_sha('e98f071751bd77f59967bfa671cd2caebdccc9a2')
         entries = [(entry1_sha, 0xf2972d0830529b87, 24),
-                   (entry2_sha, (~0xf2972d0830529b87)&(2**64-1), 92)]
+                   (entry2_sha, (~0xf2972d0830529b87) & (2 ** 64 - 1), 92)]
         if not self._supports_large:
             self.assertRaises(TypeError, self.index, 'single.idx',
-                entries, pack_checksum)
+                              entries, pack_checksum)
             return
         idx = self.index('single.idx', entries, pack_checksum)
         self.assertEqual(idx.get_pack_checksum(), pack_checksum)
@@ -665,7 +713,8 @@ class ReadZlibTests(TestCase):
     def setUp(self):
         super(ReadZlibTests, self).setUp()
         self.read = BytesIO(self.comp + self.extra).read
-        self.unpacked = UnpackedObject(Tree.type_num, None, len(self.decomp), 0)
+        self.unpacked = UnpackedObject(
+                Tree.type_num, None, len(self.decomp), 0)
 
     def test_decompress_size(self):
         good_decomp_len = len(self.decomp)
@@ -931,7 +980,8 @@ class DeltaChainIteratorTests(TestCase):
         n = 100
         objects_spec = [(Blob.type_num, b'blob')]
         for i in range(n):
-            objects_spec.append((OFS_DELTA, (i, b'blob' + str(i).encode('ascii'))))
+            objects_spec.append(
+                    (OFS_DELTA, (i, b'blob' + str(i).encode('ascii'))))
         f = BytesIO()
         entries = build_pack(f, objects_spec)
         self.assertEntriesMatch(range(n + 1), entries, self.make_pack_iter(f))
@@ -940,7 +990,8 @@ class DeltaChainIteratorTests(TestCase):
         n = 100
         objects_spec = [(Blob.type_num, b'blob')]
         for i in range(n):
-            objects_spec.append((OFS_DELTA, (0, b'blob' + str(i).encode('ascii'))))
+            objects_spec.append(
+                    (OFS_DELTA, (0, b'blob' + str(i).encode('ascii'))))
         f = BytesIO()
         entries = build_pack(f, objects_spec)
         self.assertEntriesMatch(range(n + 1), entries, self.make_pack_iter(f))
@@ -1007,8 +1058,7 @@ class DeltaChainIteratorTests(TestCase):
     def test_bad_ext_ref_non_thin_pack(self):
         blob, = self.store_blobs([b'blob'])
         f = BytesIO()
-        entries = build_pack(f, [(REF_DELTA, (blob.id, b'blob1'))],
-                             store=self.store)
+        build_pack(f, [(REF_DELTA, (blob.id, b'blob1'))], store=self.store)
         pack_iter = self.make_pack_iter(f, thin=False)
         try:
             list(pack_iter._walk_all_chains())
@@ -1050,5 +1100,7 @@ class EncodeCopyOperationTests(TestCase):
     def test_basic(self):
         self.assertEqual(b'\x80', _encode_copy_operation(0, 0))
         self.assertEqual(b'\x91\x01\x0a', _encode_copy_operation(1, 10))
-        self.assertEqual(b'\xb1\x64\xe8\x03', _encode_copy_operation(100, 1000))
-        self.assertEqual(b'\x93\xe8\x03\x01', _encode_copy_operation(1000, 1))
+        self.assertEqual(b'\xb1\x64\xe8\x03',
+                         _encode_copy_operation(100, 1000))
+        self.assertEqual(b'\x93\xe8\x03\x01',
+                         _encode_copy_operation(1000, 1))
