@@ -58,13 +58,15 @@ class ReflogLineTests(TestCase):
                 1446552482, 0, b'clone: from git://jelmer.uk/samba'))
 
     def test_parse(self):
+        reflog_line = (
+                 b'0000000000000000000000000000000000000000 '
+                 b'49030649db3dfec5a9bc03e5dde4255a14499f16 Jelmer Vernooij '
+                 b'<jelmer@jelmer.uk> 1446552482 +0000	'
+                 b'clone: from git://jelmer.uk/samba'
+                 )
         self.assertEqual(
                 (b'0000000000000000000000000000000000000000',
                  b'49030649db3dfec5a9bc03e5dde4255a14499f16',
                  b'Jelmer Vernooij <jelmer@jelmer.uk>',
                  1446552482, 0, b'clone: from git://jelmer.uk/samba'),
-                 parse_reflog_line(
-                     b'0000000000000000000000000000000000000000 '
-                     b'49030649db3dfec5a9bc03e5dde4255a14499f16 Jelmer Vernooij '
-                     b'<jelmer@jelmer.uk> 1446552482 +0000	'
-                     b'clone: from git://jelmer.uk/samba'))
+                parse_reflog_line(reflog_line))
