@@ -1,21 +1,22 @@
 # utils.py -- Test utilities for Dulwich.
 # Copyright (C) 2010 Google, Inc.
 #
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License
-# as published by the Free Software Foundation; version 2
-# of the License or (at your option) any later version of
-# the License.
+# Dulwich is dual-licensed under the Apache License, Version 2.0 and the GNU
+# General Public License as public by the Free Software Foundation; version 2.0
+# or (at your option) any later version. You can redistribute it and/or
+# modify it under the terms of either of these two licenses.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+# You should have received a copy of the licenses; if not, see
+# <http://www.gnu.org/licenses/> for a copy of the GNU General Public License
+# and <http://www.apache.org/licenses/LICENSE-2.0> for a copy of the Apache
+# License, Version 2.0.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-# MA  02110-1301, USA.
 
 """Utility functions common to Dulwich tests."""
 
@@ -23,7 +24,6 @@
 import datetime
 import os
 import shutil
-import sys
 import tempfile
 import time
 import types
@@ -50,9 +50,9 @@ from dulwich.pack import (
     create_delta,
     )
 from dulwich.repo import Repo
-from dulwich.tests import (
-    SkipTest,
+from dulwich.tests import (  # noqa: F401
     skipIf,
+    SkipTest,
     )
 
 
@@ -92,7 +92,8 @@ def make_object(cls, **attrs):
     """Make an object for testing and assign some members.
 
     This method creates a new subclass to allow arbitrary attribute
-    reassignment, which is not otherwise possible with objects having __slots__.
+    reassignment, which is not otherwise possible with objects having
+    __slots__.
 
     :param attrs: dict of attributes to set on the new object.
     :return: A newly initialized object of type cls.
@@ -101,9 +102,9 @@ def make_object(cls, **attrs):
     class TestObject(cls):
         """Class that inherits from the given class, but without __slots__.
 
-        Note that classes with __slots__ can't have arbitrary attributes monkey-
-        patched in, so this is a class that is exactly the same only with a
-        __dict__ instead of __slots__.
+        Note that classes with __slots__ can't have arbitrary attributes
+        monkey-patched in, so this is a class that is exactly the same only
+        with a __dict__ instead of __slots__.
         """
         pass
     TestObject.__name__ = 'TestObject_' + cls.__name__
@@ -125,7 +126,7 @@ def make_commit(**attrs):
     :param attrs: dict of attributes to overwrite from the default values.
     :return: A newly initialized Commit object.
     """
-    default_time = int(time.mktime(datetime.datetime(2010, 1, 1).timetuple()))
+    default_time = 1262304000  # 2010-01-01 00:00:00
     all_attrs = {'author': b'Test Author <test@nodomain.com>',
                  'author_time': default_time,
                  'author_timezone': 0,
@@ -286,15 +287,15 @@ def build_commit_graph(object_store, commit_spec, trees=None, attrs=None):
 
     :param object_store: An ObjectStore to commit objects to.
     :param commit_spec: An iterable of iterables of ints defining the commit
-        graph. Each entry defines one commit, and entries must be in topological
-        order. The first element of each entry is a commit number, and the
-        remaining elements are its parents. The commit numbers are only
+        graph. Each entry defines one commit, and entries must be in
+        topological order. The first element of each entry is a commit number,
+        and the remaining elements are its parents. The commit numbers are only
         meaningful for the call to make_commits; since real commit objects are
         created, they will get created with real, opaque SHAs.
     :param trees: An optional dict of commit number -> tree spec for building
-        trees for commits. The tree spec is an iterable of (path, blob, mode) or
-        (path, blob) entries; if mode is omitted, it defaults to the normal file
-        mode (0100644).
+        trees for commits. The tree spec is an iterable of (path, blob, mode)
+        or (path, blob) entries; if mode is omitted, it defaults to the normal
+        file mode (0100644).
     :param attrs: A dict of commit number -> (dict of attribute -> value) for
         assigning additional values to the commits.
     :return: The list of commit objects created.
@@ -360,5 +361,3 @@ def setup_warning_catcher():
         warnings.showwarning = original_showwarning
 
     return caught_warnings, restore_showwarning
-
-skipIfPY3 = skipIf(sys.version_info[0] == 3, "Feature not yet ported to python3.")
